"""
        Yasuharu Okamoto, 2018

"""
import pystan
from pystan import StanModel
import pickle
import random
import math
import matplotlib.pyplot as plt

def icc(t, a, b, c):
    return 1.0 / (1.0 + math.exp(-1.7 * a * (t - b - c)))

def CatResponse(t, a, b, c, K):
    res = 1
    v = random.random()
    while True:
        if v > icc(t, a, b, c[res - 1]):
            break
        res += 1
        if res >= K:
            break
    return res

def calcCor(data1, data2):
    sum1 = 0.0
    sum2 = 0.0
    n = len(data1)
    for i in range(n):
        sum1 += data1[i]
        sum2 += data2[i]
    mean1 = sum1 / n
    mean2 = sum2 / n
    ssum1 = 0.0
    ssum2 = 0.0
    xsum = 0.0
    for i in range(n):
        ssum1 += (data1[i] - mean1) ** 2
        ssum2 += (data2[i] - mean2) ** 2
        xsum += (data1[i] - mean1) * (data2[i])
    return xsum / ((ssum1 * ssum2) ** 0.5)

Nitems = 5                         #    Number of items
X = [1, 1, 1, 1, 1]                #    Responses to be set in simulation

a = [1.0, 1.0, 1.0, 1.0, 1.0]      #    Discrimination parameters
b = [-0.5, -0.3, 0.0, 0.3, 0.5]    #    Difficulty parameters
K = 2                              #    Number of response categories
C = [0.0]                          #    Category boundaries
#   K = 7
#   C = [-2.0, -1.0, -0.25, 0.25, 1.0, 2.0]

Data = {'K': K, 'Nitm': Nitems,  'C': C, 'a':a, 'b': b, 'X': X}

sm = StanModel(file = 'EstTheta.stan')
with open('model.pkl', 'wb') as f:
    pickle.dump(sm, f)

L_theta1 = []
L_theta2 = []

n_step = 1000
for step in range(n_step):
    if (step % 5 == 0):
        print('\nstep = {0}/{1}'.format(step, n_step))
    t = random.gauss(0.0, 1.0)
    for i in range(Nitems):
        X[i] = CatResponse(t, a[i], b[i], C, K)
    sm = pickle.load(open('model.pkl', 'rb'))
    fit = sm.sampling(data = Data,  n_jobs = 1)
    theta = fit['theta']
    theta.sort()
    L_theta1.append(theta[int(len(theta) / 2)])

    for i in range(Nitems):
        X[i] = CatResponse(t, a[i], b[i], C, K)
    sm = pickle.load(open('model.pkl', 'rb'))
    fit = sm.sampling(data = Data,  n_jobs = 1)
    theta = fit['theta']
    theta.sort()
    L_theta2.append(theta[int(len(theta) / 2)])
    
r = calcCor(L_theta1, L_theta2)

plt.title("Cor(theta1, theta2) = {0:>.3}".format(r))
plt.xlabel("theta1")              
plt.ylabel("theta2")               
plt.plot(L_theta1, L_theta2, 'bo')    
plt.show()

